<?php
namespace Teknisa\COZ\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Teknisa\Model\Planejamento\Entities\Planproducao;
// use \Teknisa\Model\Planejamento\Entities\Planprodacabado;
use \Teknisa\Model\Planejamento\Entities\Planprodfiliagrega;
use \Teknisa\Model\Planejamento\Entities\Planrecprod;


class GeracaoProdutoAcabadoService {

    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $novoCodigo;
    protected $lista;
    
    public function __construct($dataSourceManager, $environment, $entityManager, $novoCodigo, $lista) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        $this->lista = $lista;
    }

    public function geraProdutoAcabado($nrorg, $nrplanprod, $nrlistaProdAcabado, $central) {
        $planprod = $this->retornaPlanproducao($nrorg, $nrplanprod);
        
        $dtretirada = $planprod->getDtproducao();
        $dtretirada = $dtretirada->format('d/m/Y');
        $cdoperador = $this->environment->getCdOperador();
        $cdfiliproc = $this->environment->getCdFilial();
        $dslancamento = 'Retirada da matéria prima para os produtos acabados do dia.';
        $dslanctoestoq = 'Geração de produtos acabados planejados para o dia.';
        $operacao = 'G';
        
        
        $prodInativo = $this->retornaProdutoInativo($nrorg, $central, $nrplanprod, $nrlistaProdAcabado);
        if($prodInativo){
            $retorno = [];
            foreach ($prodInativo as $dado){
                $msg = $dado['CDARVPROD'] . ' - ' . $dado['NMPRODUTO'] . ': ';
                
                if($dado['IDPRODATIV'] == null || $dado['IDPRODATIV'] != 'A' || $dado['IDPRODATIVPR'] != 'S') {
                    $msg .= 'Produto inativo. ';
                }
                
                if($dado['IDCNTRESTOQ'] == null || $dado['IDCNTRESTOQ'] != 'S'){
                    $msg .= 'Não controla estoque.';
                }
                
                
                array_push($retorno, $msg);
            }
            
            if (!empty($retorno)) {
                $this->deleteLista($nrlistaProdAcabado, $nrorg);
                return $retorno;
            }
        }
        
        $retorno = $this->concluiProdAcabado($nrorg, $nrplanprod, $dtretirada, $cdoperador, $cdfiliproc, 
            $dslancamento, $dslanctoestoq, $operacao, $nrlistaProdAcabado);
        
        $this->deleteLista($nrlistaProdAcabado, $nrorg);

        if (!empty($retorno['P_RETORNO'])) {
            $msg = $this->processaMsgErro($retorno['P_RETORNO']);
            return $msg;
        }

        return true;
    }

    public function cancelaGeracaoProdutoAcabado($nrorg, $nrplanprod, $nrlistaProdAcabado, $excluiParcial) {
        $planprod = $this->retornaPlanproducao($nrorg, $nrplanprod);
        $nrretirada = $planprod->getNrretirada();

        if (empty($nrretirada)){
            $msg = 'Produtos acabados ainda não foram gerados.';
            return $msg;
        }

        $dtretirada = $planprod->getDtproducao();
        $dtretirada = $dtretirada->format('d/m/Y');
        $cdoperador = $this->environment->getCdOperador();
        $cdfiliproc = $this->environment->getCdFilial();
        $dslancamento = "";
        $dslanctoestoq = "";
        $operacao = 'C';

        $retorno = $this->concluiProdAcabado($nrorg, $nrplanprod, $dtretirada, $cdoperador, $cdfiliproc, 
            $dslancamento, $dslanctoestoq, $operacao, $nrlistaProdAcabado, $excluiParcial);

        $this->deleteLista($nrlistaProdAcabado, $nrorg);

        if (!empty($retorno['P_RETORNO'])) {
            $msg = $this->processaMsgErro($retorno['P_RETORNO']);
            return $msg;
        }

        return true;
    }

    public function retornaPlanproducao($nrorg, $nrplanprod) {
        $planproducao = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Planproducao')
            ->findOneBy(array('nrorg' => $nrorg, 'nrplanprod' => $nrplanprod));

        return $planproducao;
    }

    public function concluiProdAcabado($nrorg, $nrplanprod, $dtretirada, $cdoperador, $cdfiliproc, 
        $dslancamento, $dslanctoestoq, $operacao, $listaprodacabado, $excluiParcial = 'S') {
        
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'CONCLUI_PROD_ACABADO');
        $procedure->addParam(new Param('P_NRPLANPROD', Param::PARAM_INPUT, $nrplanprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTRETIRADA', Param::PARAM_INPUT, $dtretirada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $cdfiliproc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSLANCAMENTO', Param::PARAM_INPUT, $dslancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSLANCTOESTOQ', Param::PARAM_INPUT, $dslanctoestoq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, $operacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAPROD', Param::PARAM_INPUT, $listaprodacabado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EXCLUIPARCIAL', Param::PARAM_INPUT, $excluiParcial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 8002));
        //4009 permite retornar 242 produtos acabados mais o identificador INF_CARDJACONCL

        return $procedure->execute();
    }

    public function processaMsgErro($msg){
        if ((substr($msg,0,4) != 'ERR_') && (substr($msg,0,4) != 'INF_')){
            $mensagem = $msg;
        }
        else{
            $s = explode(';', $msg);
            $mensagem = array(
              "MENSAGEM" => "",
              "PARAMS" => []
            );
            
            if ($s[0] == 'ERR_INV_PEND'){
                //Inventário pendente. Almoxarifado: __PARAMETRO__ Local: __PARAMETRO2__
                $mensagem["MENSAGEM"] = 'ERR_INV_PEND';
                $mensagem["PARAMS"] = [$s[1], $s[2]];
            }
            elseif ($s[0] == 'ERR_EST_NEG'){
                //Operação bloqueada. Lançamento refletirá em estoque negativo. Produto: __PARAMETRO__
                $mensagem["MENSAGEM"] = 'ERR_EST_NEG';
                $mensagem["PARAMS"] = [$s[1]];
            }
            elseif($s[0] == 'ERR_N_HA_PLANPROD'){
                //Não há planejamento da produção para parâmetros informados.
                $mensagem["MENSAGEM"] = 'ERR_N_HA_PLANPROD';
            }
            elseif($s[0] == 'ERR_N_HA_PRODACABADO_CONCL'){
                //Não há produto acabado marcado para geração.
                $mensagem["MENSAGEM"] = 'ERR_N_HA_PRODACABADO_CONCL';
            }
            elseif($s[0] == 'ERR_PROD_INATIVO'){
                //Produto __PARAMETRO__ inativo na central.
                $mensagem["MENSAGEM"] = 'ERR_PROD_INATIVO';
                $mensagem["PARAMS"] = [$s[1]];
            }
            elseif($s[0] == 'ERR_CTRL_ESTOQ'){
                //Unidade não utiliza controle de estoque. Unidade: __PARAMETRO__
                $mensagem["MENSAGEM"] = 'ERR_CTRL_ESTOQ';
                $mensagem["PARAMS"] = [$s[2]];
            }
            elseif($s[0] == 'ERR_EFET_FATU'){
                //Operação bloqueada. Já existem efetivos faturados.
                $mensagem["MENSAGEM"] = 'ERR_EFET_FATU';
            }
            elseif ($s[0] == 'ERR_DIST_EFET'){
                //Existe distribuição efetivada. Produto: __PARAMETRO__
                $mensagem["MENSAGEM"] = 'ERR_DIST_EFET';
                $mensagem["PARAMS"] = [$s[1]];
            }
            elseif($s[0] == 'INF_PROD_N_ATU'){
                //Atenção! Alguns produtos não foram atualizados no estoque, pois iriam tornar o estoque irregular.
                $mensagem["MENSAGEM"] = 'INF_PROD_N_ATU';
            }
            elseif($s[0] == 'INF_CARDJACONCL'){
                $mensagem["MENSAGEM"]  = $msg;
            }
        }
        
        return $mensagem;
    }

    public function retornaPlanprodacabado($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idestado, $idfinalizacao, $idtpprodchill) {
        $planprodacabado = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Planprodacabado')
            ->findOneBy(array('nrorg' => $nrorg, 'nrplanprod' => $nrplanprod, 'cdprodacabado' => $cdprodacabado, 'nrseqprodacabado' => $nrseqprodacabado,
                'idcookchill' => $idcookchill, 'idestado' => $idestado, 'idfinalizacao' => $idfinalizacao, 'idtpprodchill' => $idtpprodchill));

        return $planprodacabado;
    }

    public function retornaPlanprodfiliagregaFK($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado) {
        $planprodfiliagrega = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Planprodfiliagrega')
            ->findBy(array('nrorg' => $nrorg, 'nrplanprod' => $nrplanprod, 'cdprodacabado' => $cdprodacabado, 'nrseqprodacabado' => $nrseqprodacabado,
                'idcookchill' => $idcookchill, 'idestado' => $idestado, 'idfinalizacao' => $idfinalizacao, 'idtpprodchill' => $idtpprodchill));

        return $planprodfiliagrega;
    }

    public function retornaPlanrecprodFK($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $cdfiliagreg, $cdservfiliagreg, $dtconsumo, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado) {
        $planrecprod = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Planrecprod')
            ->findBy(array('nrorg' => $nrorg, 'nrplanprod' => $nrplanprod, 'cdprodacabado' => $cdprodacabado, 'nrseqprodacabado' => $nrseqprodacabado, 'cdfiliagreg' => $cdfiliagreg,
                'cdservfiliagreg' => $cdservfiliagreg, 'idcookchill' => $idcookchill, 'dtconsumo' => $dtconsumo, 'idestado' => $idestado, 'idfinalizacao' => $idfinalizacao, 'idtpprodchill' => $idtpprodchill));

        return $planrecprod;
    }

    public function montaListaProdutoAcabado($nrorg, $produtosAcabados){
        $produtosAcabados = rtrim($produtosAcabados, '_');
        $arrayProdutosAcabados = explode('_', $produtosAcabados);
         $nrlistaProdutoAcabado = $this->novoCodigo->getCodeByTableName("LISTA", $nrorg);
        foreach($arrayProdutosAcabados as $prodAcab){
            $ProdutoAcabadoParametros = array(
                "NRORG" => $nrorg, "NRLISTA" => $nrlistaProdutoAcabado, "CDLISTA" => $prodAcab
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '', "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '', "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '', "DTULTATUPRFIL" => ''
                , "__is_new" => true
            );
            $this->saveLista($ProdutoAcabadoParametros);
        }
        return $nrlistaProdutoAcabado;
    }

    public function saveLista($param){
        $request = new DTO\Request\Row($param, "save", "/dummy", " ");
        $response = new DTO\Response();
        $this->lista->save($request, $response);
    }

    public function deleteLista($nrLista, $nrorg=1){
        $connection = $this->entityManager->getConnection();
        $query="DELETE FROM LISTA WHERE NRLISTA = :NRLISTA";
        $connection->executeQuery($query, array("NRLISTA"=>$nrLista));
    }

    public function salvaEdicaoProdAcabado($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idfinalizacao, $idtpprodchill, $idestado, $qtextra, $qtperda, $cdcontainer){
        $planprodacabado = $this->retornaPlanprodacabado($nrorg, $nrplanprod, $cdprodacabado, $nrseqprodacabado, $idcookchill, $idestado, $idfinalizacao, $idtpprodchill);

        $planprodacabado->setQtextra($qtextra);
        $planprodacabado->setQtperda($qtperda);
        $planprodacabado->setCdcontainer($cdcontainer);

        $this->entityManager->persist($planprodacabado);
        $this->entityManager->flush();

        return true;
    }

    public function salvaEdicaoProdFiliAgrega($planprodfiliagrega, $qtextra, $qtperda){
        $planprodfiliagrega->setQtextra($qtextra);
        $planprodfiliagrega->setQtperda($qtperda);

        $planprodfiliagrega->setVrtotalprodserv(($planprodfiliagrega->getQtaproduzir() + $qtextra) * $planprodfiliagrega->getVrunitprodserv());

        $this->entityManager->persist($planprodfiliagrega);
        $this->entityManager->flush();

        return true;
    }

    public function salvaEdicaoRecProd($planrecprod, $qtextra){
        $planrecprod->setQtextra($qtextra);
        $planrecprod->setVrtotalreceprat(($planrecprod->getQtaproduzir() + $qtextra) * $planrecprod->getVrunitreceprat());

        $this->entityManager->persist($planrecprod);
        $this->entityManager->flush();

        return true;
    }
    
    public function retornaProdutoInativo($nrorg, $central, $nrplanprod, $nrlista){
        $sql = "SELECT P.CDPRODACABADO, PT.CDARVPROD, PT.NMPRODUTO, P.QTPRODACABADO, PT.VRFATOCONV,  
                       PF.IDPRODATIV, PF.IDCNTRESTOQ, PF.IDUTCTRLLOTE, 
                       PT.CDPRODESTO, PT.IDPRODATIVPR
                  FROM PLANPRODACABADO P 
                       INNER JOIN PRODUTO PT ON PT.CDPRODUTO = P.CDPRODACABADO 
                       LEFT JOIN PRODFILI PF ON PF.CDPRODUTO = PT.CDPRODUTO AND PF.CDFILIAL =  :P_CDCENTRAL
                       INNER JOIN LISTA L_PROD ON ((NRLISTA         = :P_LISTAPROD)        AND 
                                                   (P.CDPRODACABADO = SUBSTR(L_PROD.CDLISTA,1,10))AND 
                                                   (P.IDCOOKCHILL   = SUBSTR(L_PROD.CDLISTA,11,1))AND 
                                                   (P.IDFINALIZACAO = SUBSTR(L_PROD.CDLISTA,12,1))AND 
                                                   (SUBSTR(P.IDTPPRODCHILL,1,1) = SUBSTR(L_PROD.CDLISTA,13,1))AND 
                                                   (SUBSTR(P.IDESTADO,1,1)      = SUBSTR(L_PROD.CDLISTA,14,1)) AND 
                                                   (P.IDESTADO <> 'FINALIZACAO') AND 
                                                   (P.NRSEQPRODACABADO          = SUBSTR(L_PROD.CDLISTA,15)))      
                 WHERE P.NRPLANPROD = :P_NRPLANPROD 
                   AND P.IDPRODUZIR = 'S' 
                   AND (PF.IDPRODATIV = 'I' OR PT.IDPRODATIVPR = 'N')";


        $params = array(
            'P_NRPLANPROD' => $nrplanprod,
            'P_CDCENTRAL' => $central,
            'P_LISTAPROD' => $nrlista
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}